#include "main.h"
#include <stdio.h>

void op_br(u16 instr) {
  u16 pc_offset = sign_extend(instr & 0x1FF, 9);
  u16 cond_flag = (instr >> 9) & 0x7;
  if (cond_flag & reg[R_COND]) {
    reg[R_PC] += pc_offset;
  }
}

void op_add(u16 instr) {
  u16 dr = (instr >> 9) & 0x7; // 0x7 = 0b111
  u16 sr1 = (instr >> 6) & 0x7;
  u16 immediate_mode = (instr >> 5) & 0x1;

  if (immediate_mode) {
    u16 imm5 = sign_extend(instr & 0x1F, 5);
    reg[dr] = reg[sr1] + imm5;
  } else {
    u16 sr2 = instr & 0x7;
    reg[dr] = reg[sr1] + reg[sr2];
  }

  update_flags(dr);
}

void op_ld(u16 instr) {
  u16 dr = (instr >> 9) & 0x7;
  u16 pc_offset = instr & 0x1FF;

  reg[dr] = mem_read(reg[R_PC] + sign_extend(pc_offset, 9));
  update_flags(dr);
}

void op_st(u16 instr) {
  u16 sr = (instr >> 9) & 0x7;
  u16 pc_offset = instr & 0x1FF;
  
  mem_write(reg[R_PC] + sign_extend(pc_offset, 9), reg[sr]);
}

void op_jsr(u16 instr) {
  u16 long_flag = (instr >> 11) & 0x1;
  reg[R_R7] = reg[R_PC];
  if (long_flag) {
    reg[R_PC] += sign_extend(instr & 0x7FF, 11); // eleven set bits
  } else {
    reg[R_PC] = reg[(instr >> 6) & 0x7];
  }
}

void op_and(u16 instr) {
  u16 dr = (instr >> 9) & 0x7;
  u16 sr1 = (instr >> 6) & 0x7;
  u16 immediate_mode = (instr >> 5) & 0x1;

  if (immediate_mode) {
    u16 imm5 = sign_extend(instr & 0x1F, 5);
    reg[dr] = reg[sr1] & imm5;
  } else {
    u16 sr2 = instr & 0x7;
    reg[dr] = reg[sr1] & reg[sr2];
  }

  update_flags(dr);
}

void op_ldr(u16 instr) {
  u16 dr = (instr >> 9) & 0x7;
  u16 baser = (instr >> 6) & 0x7;
  u16 offset = sign_extend(instr & 0x3F, 6);

  reg[dr] = mem_read(reg[baser] + offset);
  update_flags(dr);
}

void op_str(u16 instr) {
  u16 sr = (instr >> 9) & 0x7;
  u16 baser = (instr >> 6) & 0x7;
  u16 offset = sign_extend(instr & 0x3F, 6);

  mem_write(reg[baser] + offset, reg[sr]);
  
}

void op_not(u16 instr) {
  u16 dr = (instr >> 9) & 0x7;
  u16 sr = (instr >> 6) & 0x7;

  reg[dr] = ~(reg[sr]);
  update_flags(dr);
}

void op_ldi(u16 instr) {
  u16 dr = (instr >> 9) & 0x7;
  u16 pc_offset = sign_extend(instr & 0x1FF, 9);

  reg[dr] = mem_read(mem_read(reg[R_PC] + pc_offset));
  update_flags(dr);
}

void op_sti(u16 instr) {
  u16 sr = (instr >> 9) & 0x7;
  u16 pc_offset = sign_extend(instr & 0x1FF, 9);
  
  mem_write(mem_read(reg[R_PC] + pc_offset), reg[sr]);
}

void op_jmp(u16 instr) {
  u16 baser = (instr >> 6) & 0x7;
  
  reg[R_PC] = reg[baser];
}

void op_lea(u16 instr) {
  u16 dr = (instr >> 9) & 0x7;
  u16 pc_offset = sign_extend(instr & 0x1FF, 9);

  reg[dr] = reg[R_PC] + pc_offset;
  update_flags(dr);
}
